package drools.test;

import java.io.InputStreamReader;

import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.StatefulSession;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderConfiguration;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;


public class DroolsTest {

	private static RuleBase ruleBase;

	private StatefulSession workingMemory;

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		final PackageBuilderConfiguration config = new PackageBuilderConfiguration();
		config.setJavaLanguageLevel("1.5");
		final PackageBuilder builder = new PackageBuilder(config);
		builder
				.addPackageFromDrl(new InputStreamReader(
						Thread
								.currentThread()
								.getContextClassLoader()
								.getResourceAsStream(
										"test.drl")));

		ruleBase = RuleBaseFactory.newRuleBase();
		ruleBase.addPackage(builder.getPackage());
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
		workingMemory = ruleBase.newStatefulSession();
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
		workingMemory.dispose();
	}
	
	@Test
	public final void test() throws Exception {
		String item1 = "1";
		String item2 = "2";
		String item3 = "3";
		String item4 = "4";
		String item5 = "5";
		
		FirstClass f = new FirstClass();
		f.setItem1(item1);
		f.setItem2(item2);
		f.setItem3(item3);
		f.setItem4(item4);
		f.setItem5(item5);
		
		SecondClass s1 = create(null,  item2, item3, item4, item5);
		SecondClass s2 = create(item1, null,  item3, item4, item5);
		SecondClass s3 = create(item1, item2, null,  item4, item5);
		SecondClass s4 = create(item1, item2, item3, null,  item5);
		SecondClass s5 = create(item1, item2, item3, item4, null);
		
		workingMemory.insert(s1);
		workingMemory.insert(s2);
		workingMemory.insert(s3);
		workingMemory.insert(s4);
		// comment out the insertion of s5 and the correct NOT rule will fire
		// otherwise the equality will fire...when they are NOT equal
		// you can also change the equality check order, so that item5 is in the first position
		// and item1 is in the fifth.  Then s1 will cause the EQUALITY rule to fire incorrectly.
		workingMemory.insert(s5);
		workingMemory.insert(f);
		
		workingMemory.fireAllRules();
		
		
		
	}

	private SecondClass create(String item1, String item2, String item3,
			String item4, String item5) {
		SecondClass c = new SecondClass();
		c.setItem1(item1);
		c.setItem2(item2);
		c.setItem3(item3);
		c.setItem4(item4);
		c.setItem5(item5);
		return c;
	}
}
